#include "lppmonplot.h"



LppMonPlot::LppMonPlot(QWidget *parent) :
    QWidget(parent)
{
    this->m_plot = new QCustomPlot(this);
    this->m_plot->setInteractions(QCustomPlot::iRangeDrag  | QCustomPlot::iSelectAxes |
                                  QCustomPlot::iSelectLegend | QCustomPlot::iSelectPlottables | QCustomPlot::iSelectTitle);
    this->m_plot->setRangeDrag(Qt::Horizontal|Qt::Vertical);
    this->m_plot->setRangeZoom(Qt::Horizontal|Qt::Vertical);
    this->m_mainlayout = new QGridLayout(this);
    this->setLayout(this->m_mainlayout);
    this->m_mainlayout->addWidget(this->m_plot);
    this->setMinimumSize(400,300);
    this->setFocusPolicy(Qt::WheelFocus);
    this->m_plot->setAttribute(Qt::WA_TransparentForMouseEvents);
    this->ctrl_hold = false;
    this->shift_hold = false;
    this->mouse_hold = false;
    this->show();

}

void LppMonPlot::show()
{
    QWidget::show();
}

void LppMonPlot::setTitle(QString title)
{
    this->m_plot->setTitle(title);
    this->repaint();
}

void LppMonPlot::setXaxisLabel(QString label)
{
    this->m_plot->xAxis->setLabel(label);
    this->repaint();
}

void LppMonPlot::setYaxisLabel(QString label)
{
    this->m_plot->yAxis->setLabel(label);
    this->repaint();
}

void LppMonPlot::setXaxisRange(double lower, double upper)
{
    this->m_plot->xAxis->setRange(lower,upper);
}

void LppMonPlot::setYaxisRange(double lower, double upper)
{
    this->m_plot->yAxis->setRange(lower,upper);
}


void LppMonPlot::rescaleAxis()
{
    this->m_plot->rescaleAxes();
    this->m_plot->replot();
}

void LppMonPlot::setLegendFont(QFont font)
{
    this->m_plot->legend->setFont(font);
    this->repaint();
}

void LppMonPlot::setLegendSelectedFont(QFont font)
{
    this->m_plot->legend->setSelectedFont(font);
    this->repaint();
}

int LppMonPlot::addGraph()
{
    this->m_plot->addGraph();
    return this->m_plot->graphCount() -1;
}


void LppMonPlot::setGraphName(int graphIndex,QString name)
{
    if(graphIndex<this->m_plot->graphCount())
    {
        this->m_plot->graph(graphIndex)->setName(name);
    }
}


void LppMonPlot::setGraphData(int graphIndex, QList<QVariant> x, QList<QVariant> y)
{
    if((graphIndex<this->m_plot->graphCount()) && (x.count()==y.count()))// && (x.at(0).type()==QVariant::Double))
    {
        QVector<double> _x(x.count()), _y(y.count());
        for(int i=0;i<x.count();i++)
        {
            /*_x[i] = x.at(i).value<double>();
            _y[i] = y.at(i).value<double>();*/
            _x[i] = x.at(i).toDouble();
            _y[i] = y.at(i).toDouble();
        }
        this->m_plot->graph(graphIndex)->setData(_x,_y);
    }
    this->m_plot->replot();
}

void LppMonPlot::addGraphData(int graphIndex, QList<QVariant> x, QList<QVariant> y)
{
    if((graphIndex<this->m_plot->graphCount()) && (x.count()==y.count()))// && (x.at(0).type()==QVariant::Double))
    {
        QVector<double> _x(x.count()), _y(y.count());
        for(int i=0;i<x.count();i++)
        {
            /*_x[i] = x.at(i).value<double>();
            _y[i] = y.at(i).value<double>();*/
            _x[i] = x.at(i).toDouble();
            _y[i] = y.at(i).toDouble();
        }
        this->m_plot->graph(graphIndex)->addData(_x,_y);
    }
    this->m_plot->replot();
}

void LppMonPlot::addGraphData(int graphIndex, QVariant x, QVariant y)
{
    if(graphIndex<this->m_plot->graphCount())// && (x.at(0).type()==QVariant::Double))
    {
        this->m_plot->graph(graphIndex)->addData(x.toDouble(),y.toDouble());
    }
    this->m_plot->replot();
}

void LppMonPlot::setGraphPen(int graphIndex,QPen pen)
{
    if(graphIndex<this->m_plot->graphCount())
    {
        this->m_plot->graph(graphIndex)->setPen(pen);
    }
}

QPen LppMonPlot::getGraphPen(int graphIndex)
{
    if(graphIndex<this->m_plot->graphCount())
    {
        return this->m_plot->graph(graphIndex)->pen();
    }
}



void LppMonPlot::setGraphLineStyle(int graphIndex,QString lineStyle)
{
    if(graphIndex<this->m_plot->graphCount())
    {
        if(!lineStyle.compare("none"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsNone);
            return;
        }
        if(!lineStyle.compare("line"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsLine);
            return;
        }
        if(!lineStyle.compare("stepleft"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsStepLeft);
            return;
        }
        if(!lineStyle.compare("stepright"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsStepRight);
            return;
        }
        if(!lineStyle.compare("stepcenter"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsStepCenter);
            return;
        }
        if(!lineStyle.compare("impulse"))
        {
            this->m_plot->graph(graphIndex)->setLineStyle(QCPGraph::lsImpulse);
            return;
        }


    }
}

void LppMonPlot::setGraphScatterStyle(int graphIndex,QString scatterStyle)
{
    if(graphIndex<this->m_plot->graphCount())
    {
        if(!scatterStyle.compare("none"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssNone);
            return;
        }
        if(!scatterStyle.compare("dot"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssDot);
            return;
        }
        if(!scatterStyle.compare("cross"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssCross);
            return;
        }
        if(!scatterStyle.compare("plus"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssPlus);
            return;
        }
        if(!scatterStyle.compare("circle"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssCircle);
            return;
        }
        if(!scatterStyle.compare("disc"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssDisc);
            return;
        }
        if(!scatterStyle.compare("square"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssSquare);
            return;
        }
        if(!scatterStyle.compare("diamond"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssDiamond);
            return;
        }
        if(!scatterStyle.compare("star"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssStar);
            return;
        }
        if(!scatterStyle.compare("triangle"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssTriangle);
            return;
        }
        if(!scatterStyle.compare("invertedtriangle"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssTriangleInverted);
            return;
        }
        if(!scatterStyle.compare("crosssquare"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssCrossSquare);
            return;
        }
        if(!scatterStyle.compare("plussquare"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssPlusSquare);
            return;
        }
        if(!scatterStyle.compare("crosscircle"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssCrossCircle);
            return;
        }
        if(!scatterStyle.compare("pluscircle"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssPlusCircle);
            return;
        }
        if(!scatterStyle.compare("peace"))
        {
            this->m_plot->graph(graphIndex)->setScatterStyle(QCP::ssPeace);
            return;
        }

    }
}





void LppMonPlot::keyPressEvent(QKeyEvent * event)
{
    switch(event->key())
    {
    case Qt::Key_Control:
        this->ctrl_hold = true;
        break;
    case Qt::Key_Shift:
        this->shift_hold = true;
        break;
    case Qt::Key_M:
        this->rescaleAxis();
        break;
    default:
        QWidget::keyPressEvent(event);
        break;
    }
}

void LppMonPlot::keyReleaseEvent(QKeyEvent * event)
{
    switch(event->key())
    {
    case Qt::Key_Control:
        event->accept();
        this->ctrl_hold = false;
        break;
    case Qt::Key_Shift:
        event->accept();
        this->shift_hold = false;
        break;
    default:
        QWidget::keyReleaseEvent(event);
        break;
    }
}

void LppMonPlot::wheelEvent(QWheelEvent * event)
{
    double factor;
    double wheelSteps = event->delta()/120.0; // a single step delta is +/-120 usually
    if(ctrl_hold)
    {
        if (event->orientation()==Qt::Vertical)//mRangeZoom.testFlag(Qt::Vertical))
        {
            factor = pow(this->m_plot->rangeZoomFactor(Qt::Vertical), wheelSteps);
            QCPAxis* axis = this->m_plot->rangeZoomAxis(Qt::Vertical);
            axis->scaleRange(factor, axis->pixelToCoord(event->pos().y()));
        }
        this->m_plot->replot();
        QWidget::wheelEvent(event);
        return;
    }
    if(shift_hold)
    {
        if (event->orientation()==Qt::Vertical)//mRangeZoom.testFlag(Qt::Vertical))
        {
            factor = pow(this->m_plot->rangeZoomFactor(Qt::Horizontal), wheelSteps);
            QCPAxis* axis = this->m_plot->rangeZoomAxis(Qt::Horizontal);
            axis->scaleRange(factor, axis->pixelToCoord(event->pos().x()));
        }
        this->m_plot->replot();
        QWidget::wheelEvent(event);
        return;
    }
    QCPAxis* Haxis = this->m_plot->rangeDragAxis(Qt::Horizontal);
    double rg = (Haxis->range().upper - Haxis->range().lower)*(wheelSteps/10);
    Haxis->setRange(Haxis->range().lower+(rg), Haxis->range().upper+(rg));
    this->m_plot->replot();
    QWidget::wheelEvent(event);
}




void LppMonPlot::mousePressEvent(QMouseEvent *event)
{
    if(event->button()==Qt::LeftButton)
    {
        mDragStart = event->pos();
        this->mouse_hold = true;
        DragStartHorzRange = this->m_plot->rangeDragAxis(Qt::Horizontal)->range();
        DragStartVertRange = this->m_plot->rangeDragAxis(Qt::Vertical)->range();
    }
    QWidget::mousePressEvent(event);
}

void LppMonPlot::mouseReleaseEvent(QMouseEvent *event)
{
    if(event->button()==Qt::LeftButton)
    {
        this->mouse_hold = false;
    }
    QWidget::mouseReleaseEvent(event);
}

void LppMonPlot::mouseMoveEvent(QMouseEvent *event)
{
    if(mouse_hold)
    {
        QCPAxis* Haxis = this->m_plot->rangeDragAxis(Qt::Horizontal);
        QCPAxis* Vaxis = this->m_plot->rangeDragAxis(Qt::Vertical);
        double diff = Haxis->pixelToCoord(mDragStart.x()) - Haxis->pixelToCoord(event->pos().x());
        Haxis->setRange(DragStartHorzRange.lower+diff, DragStartHorzRange.upper+diff);
        diff = Vaxis->pixelToCoord(mDragStart.y()) - Vaxis->pixelToCoord(event->pos().y());
        Vaxis->setRange(DragStartVertRange.lower+diff, DragStartVertRange.upper+diff);
        this->m_plot->replot();
    }
    QWidget::mouseMoveEvent(event);
}











